"""
Streamlit Application
Nicolas Atrax
"""

import streamlit as st
import pandas as pd
import chardet
import csv
import re
from datetime import date
import codecs
import src.basic as tmp


tmp.base("CleanCSVtoTSV")


# Code Start

def checkEncoding(file):
    content = file.read()
    encoding = chardet.detect(content)["encoding"].lower()
    if encoding != "ascii" and encoding != "utf-8" and encoding != "utf-8-sig":
        return False
    return True


def getSeparator(file):

    line = file.readline().decode('utf-8')

    if ',' in line:
        return ',', True

    if ';' in line:
        return ';', True

    return '\t', False


def checkPublicationCase(tmp, split, success):
    if split:
        if tmp[0][0].isupper() or tmp[1][0].isupper():
            return False
        else:
            return success
    if not tmp[0][0].isupper() or not tmp[1][0].isupper():
        return False
    return success


def checkPublication(name, registeredNames, errorMessage):
    tmpName = name
    if re.search('[a-zA-Z0-9]', name[0]) == None:
        tmpName = name[1:]
    tmp = tmpName.split(' ')
    success = True
    split = False
    first = ""
    second = ""
    if "_" in tmp[0] and len(tmp) == 1:
        tmp = tmp[0].split('_')
        split = True
    if len(tmp) != 2:
        success = False
    else:
        success = checkPublicationCase(tmp, split, success)
        first = tmp[0][0].lower() + tmp[0][1:]
        second = tmp[1][0].lower() + tmp[1][1:]
        if first != "publication" or second not in ["day", "month", "year"]:
            success = False
    if not success:
        errorMessage += "Error at line 1 ! Wrong name : " + \
            name + " is not appropriated ! \n"
    else:
        registeredNames.append(first + "_" + second)
    return success, errorMessage


def checkNameValidity(name, columnNames, registeredNames, errorMessage):
    tmpName = name
    if re.search('[a-zA-Z0-9]', name[0]) == None:
        tmpName = name[1:]
    if tmpName not in columnNames:
        errorMessage += "Error at line 1 ! Wrong name : " + \
            name + " is not appropriated ! \n"
        return False, errorMessage
    if tmpName in registeredNames:
        errorMessage += "Error at line 1 ! Same name for 2 differents columns! \n"
        return False, errorMessage
    return True, errorMessage


def checkColumnExistence(registeredNames, errorMessage):
    columnNames = ["authors", "title", "publication_year",
                   "publication_month", "publication_day", "abstract", "source"]
    if len(registeredNames) != 7:
        tmpMessage = "Error at line 1 ! Following columns are missing :"
        for name in columnNames:
            if name not in registeredNames:
                tmpMessage += " " + name
        errorMessage += tmpMessage + "\n"
        return False, errorMessage
    return True, errorMessage


def checkColumnNames(name, errorMessage, registeredNames, success):
    columnNames = ["authors", "title", "publication_year",
                   "publication_month", "publication_day", "abstract", "source"]
    name = name.replace("\n", "")
    if len(name) > 9:
        tmpSuccess, errorMessage = checkPublication(
            name, registeredNames, errorMessage)
    else:
        name = name.replace(" ", "")
        tmpSuccess, errorMessage = checkNameValidity(
            name[0].lower() + name[1:], columnNames, registeredNames, errorMessage)
        if tmpSuccess:
            registeredNames.append(name[0].lower() + name[1:])
    if success:
        success = tmpSuccess
    return success, errorMessage, registeredNames


def getColumnsNames(file, separator, errorMessage):
    data = ""
    columnNb = 0
    registeredNames = []
    success = True
    reader = csv.DictReader(codecs.iterdecode(
        file, 'utf-8'), delimiter=separator)
    columnsNames = []
    for row in reader:
        for name, value in row.items():
            columnName = name.replace("\ufeff", "")
            if (columnNb < 7):
                success, errorMessage, registeredNames = checkColumnNames(
                    name, errorMessage, registeredNames, success)
            if data != "":
                data += "\t"
            data += columnName
            columnNb += 1
        success, errorMessage = checkColumnExistence(
            registeredNames, errorMessage)
        break
    data += "\n"
    return data, success, errorMessage


def lowerName(name):
    tmp = name.split(' ')
    split = False
    first = ""
    second = ""
    if len(tmp) == 1 and "_" in tmp[0]:
        tmp = tmp[0].split('_')
        split = True
    if len(tmp) != 2:
        return name.lower()
    else:
        first = tmp[0][0].lower() + tmp[0][1:]
        second = tmp[1][0].lower() + tmp[1][1:]
        return first + "_" + second


def checkDate(name, value, success, fill, csvLine, errorMessage):
    if name in ["publication_year", "publication_month", "publication_day"]:
        if value == "" or value == "\n":
            if fill:
                if name == "publication_year":
                    return success, str(date.today().year), errorMessage
                return success, "1", errorMessage
            errorMessage += name + " is missing at line " + \
                str(csvLine) + ". Please add it or replace blank space by \"1\".\n"
        if not value.replace("\n", "").isnumeric():
            errorMessage += "Error at line " + \
                str(csvLine) + ", wrong date ! Date is : \"" + \
                value.replace("\n", "") + "\". " + name + " must be numeric.\n"
            return False, "", errorMessage
        if int(value) <= 0:
            errorMessage += "Error at line " + str(csvLine) + ", wrong date ! Date is : \"" + value.replace(
                "\n", "") + "\". " + name + " can't be 0 or less.\n"
            return False, "", errorMessage
    return success, "", errorMessage


def checkMissing(name, value, success, fill, csvLine, errorMessage):
    if value == "" or value == "\n":
        if name not in ["publication_year", "publication_month", "publication_day"]:
            if fill:
                return success, "null", errorMessage
            errorMessage += "Error at line " + \
                str(csvLine) + " ! " + name + \
                " is missing. Please add it or replace the blank space with \"null\".\n"
            return False, "", errorMessage
        else:
            return checkDate(name, value, success, fill, csvLine, errorMessage)
    return success, "", errorMessage


def correctedSequence(text):
    if text == None:
        return ""
    tmp = text.replace("\"", "\"\"")
    find = "\t" in text or "\"" in text or "\n" in text or "," in text or ";" in text
    if find:
        if text[-1] == "\n":
            tmp = tmp[:-1]
        tmp = "\"" + tmp + "\""
    return tmp


def getContent(file, separator, data, success, fill, errorMessage):
    reader = csv.DictReader(codecs.iterdecode(
        file, 'utf-8'), delimiter=separator)
    csvLine = 2
    columnNb = 0
    for row in reader:
        tmp = ""
        first = True
        for name, value in row.items():
            tmpFill = ""
            if not first:
                tmp += "\t"
            else:
                first = False
            if (columnNb < 7):
                success, tmpFill, errorMessage = checkMissing(
                    lowerName(name), value, success, fill, csvLine, errorMessage)
                if tmpFill != "":
                    tmp += tmpFill
                else:
                    success, tmpFill, errorMessage = checkDate(
                        lowerName(name), value, success, fill, csvLine, errorMessage)
                    tmp += correctedSequence(value)
            else:
                tmp += correctedSequence(value)
            columnNb += 1
        columnNb = 0
        csvLine += 1
        data += tmp + "\n"
    return data[:-1], success, errorMessage


# Code End

st.write(st.session_state.general_text_dict['text'])
st.session_state.fill = st.checkbox(st.session_state.general_text_dict['fill'])
file = st.file_uploader(
    st.session_state.general_text_dict['file'], type=["tsv", "csv"], key='file')

if file:
    name = file.name.split('.')[0] + '.tsv'
    if checkEncoding(file):
        file.seek(0)
        separator, wrongSeparators = getSeparator(file)
        file.seek(0)
        st.session_state.data, st.session_state.success, st.session_state.error = getColumnsNames(
            file, separator, "")
        file.seek(0)

        if st.session_state.success:
            st.session_state.data, st.session_state.success, st.session_state.error = getContent(
                file, separator, st.session_state.data, st.session_state.success, st.session_state.fill, st.session_state.error)
            file.seek(0)

        if st.session_state.success:
            st.write(st.session_state.general_text_dict['new_file'])
            st.download_button(name, st.session_state.data, name)
        else:
            st.write(st.session_state.general_text_dict['errorMessage'])
            st.download_button("ErrorMessage.txt",
                               st.session_state.error, "ErrorMessage.txt")
    else:
        st.write(st.session_state.general_text_dict['encoding'])

    file.close()
