"""
Streamlit Application
Nicolas Atrax
"""

import streamlit as st
import pandas as pd
import chardet
import re
from datetime import date
import codecs
import os
import tempfile
from lib.langdetect.langdetect import detect
from lib.langdetect.langdetect.lang_detect_exception import LangDetectException
import src.basic as tmp

tmp.base("TXTtoTSV")

# Tool Code Start


def detectLanguages(abstract, languages):
    if re.search('[a-zA-Z]', abstract) == None:
        return languages
    try:
        tmp = detect(abstract)
        if tmp in languages:
            languages[tmp] += 1
        else:
            languages[tmp] = 1
    except LangDetectException:
        pass
    return languages


def estimateLanguagesPercentage(languages):
    total = 0
    max = 0
    principal = ""
    res = {}
    for l in languages:
        total += int(languages[l])
    for l in languages:
        tmp = int(languages[l]) / total * 100
        if tmp > max:
            max = tmp
            principal = l
        res[l] = tmp
    return res, principal


def detectMultipleLanguages(languages, fileName):
    languages, principal = estimateLanguagesPercentage(languages)
    detected = ""
    valuable = []
    for l in languages:
        if languages[l] >= 15:
            if detected != "":
                detected += "| "
            detected += l + " : " + str(languages[l]) + "%"
            valuable.append(l)
    if len(valuable) > 1:
        st.session_state.warning += st.session_state.general_text_dict['warning'] + "\"" + \
            fileName + "\" !\n"
        st.session_state.warning += st.session_state.general_text_dict['warning2'] + \
            detected + " \n"
        st.session_state.warning += st.session_state.general_text_dict['advice'] + "\n\n"
        return principal
    return principal


def segmentAbstract(abstract, tsv):
    year = str(date.today().year)
    month = "1"
    day = "1"
    source = st.session_state.title
    if source == "":
        source = st.session_state.file.name.replace('.txt', '')
    author = st.session_state.author
    nbLines = len(abstract)
    n = 0
    count = 1
    languages = {}
    while n < nbLines - 2:
        doc = "".join(abstract[n: n + 9]).replace("�", "")
        title = source + " : Part " + str(count)
        tsv += correctedSequence(author, False) + "\t" + correctedSequence(
            source, False) + "\t" + year + "\t" + month + "\t" + day + "\t"
        tsv += correctedSequence(title, False) + "\t"
        tsv += correctedSequence(doc, True)
        if tsv[-1] != "\n":
            tsv += "\n"
        n += 7
        count += 1
        if n > nbLines - 9 and n != nbLines - 2:
            n = nbLines - 9
        languages = detectLanguages(doc, languages)

    detectMultipleLanguages(languages, source)
    return tsv


def correctedSequence(text, last):
    tmp = text.replace("\"", "\"\"")
    find = "\t" in text or "\"" in text or "\n" in text
    if find:
        if text[len(text) - 1] == "\n":
            tmp = tmp[:-1]
        tmp = "\"" + tmp + "\""
        if last:
            tmp += "\n"
    return tmp


def getTxt():
    txt = []
    st.session_state.pdfDir = tempfile.TemporaryDirectory()
    name = st.session_state.file.name
    with open(st.session_state.pdfDir.name + "/" + name, "wb") as file:
        file.write(st.session_state.file.getvalue())
    with open(st.session_state.pdfDir.name + "/" + name, "r") as file:
        for line in file:
            txt.append(line)
    return txt


def txtToTSV():

    fileName = st.session_state.file.name
    txt = getTxt()
    tsv = "authors\tsource\tpublication_year\tpublication_month\tpublication_day\ttitle\tabstract\n"
    tsv = segmentAbstract(txt, tsv)

    return tsv


# Tool Code End


form = st.form('api')

# Page Code Start

if 'page' not in st.session_state:
    st.session_state.page = 0

if 'warning' not in st.session_state:
    st.session_state.warning = ""


def setSubmit():
    st.session_state.submit = True


def setPage():
    st.session_state.page = 0


def askTXT():

    with st.form("Submit"):
        st.write(st.session_state.general_text_dict['text'])
        col1, col2 = st.columns(2)
        st.session_state.author = ""
        with col1:
            st.text_input(
                st.session_state.general_text_dict['author'], key='author')
        with col2:
            st.text_input(
                st.session_state.general_text_dict['titlePDF'], key='title')
        st.file_uploader(
            st.session_state.general_text_dict['file'], type=["txt"], key='file')
        st.form_submit_button(
            st.session_state.general_text_dict['submit'], on_click=setSubmit())


# Page Code End

if 'submit' not in st.session_state:
    st.session_state.submit = False

if st.session_state.page == 0:
    if st.session_state.submit:
        if st.session_state.file != None:
            print(st.session_state.file)
            st.session_state.page = 1
        else:
            askTXT()
    else:
        askTXT()

if st.session_state.page == 1:
    name = st.session_state.file.name
    txt = txtToTSV()
    name = st.session_state.file.name.split('.')[0] + '.tsv'
    st.write(st.session_state.general_text_dict['new_file'])
    st.session_state.submit = False
    if st.session_state.warning != "":
        st.info(st.session_state.warning)
    st.download_button(name, txt, name, on_click=setPage())
